#!/usr/bin/env python
from scipy.optimize import differential_evolution
import matplotlib.pyplot as plt
from matplotlib import cm
import numpy as np

def plot_cost(known, target):
    N = 100
    [offset_x, offset_y] = np.meshgrid(np.linspace(-1,1,N),np.linspace(-1,1,N))
    C = np.zeros_like(offset_x) # cost at each offset point
    offset_target = np.zeros_like(target)
    for r in range(N):
        for c in range(N):
            offset = np.array([offset_x[r,c], offset_y[r,c]])
            offset_target = target + offset
            C[r,c] = cost(known, offset_target)
    fig, ax = plt.subplots()
    plt.contourf(offset_x, offset_y, C, levels=20)
    ax.set_aspect('equal')
    ax.set_xlim([-1,1.0])
    ax.set_xlabel('X offset')
    ax.set_ylabel('Y offset')
    ax.set_ylim([-1,1.0])
    plt.colorbar()
    plt.show()

def cost(known, target):
    dist = np.sqrt( ( target[:,0][:,np.newaxis] - known[:,0] )**2 +
                    ( target[:,1][:,np.newaxis] - known[:,1] )**2 )
    return np.min(dist, axis=0).sum()
    #          or, equivalently,
    #      np.sum(np.min(dist, axis=0))

def fn(independent_vars, *data):
    x_offset, y_offset = independent_vars
    known, target = data
    offset_target = target + np.array([x_offset, y_offset])
    return cost(known, offset_target)

def main():
    bounds = ([-1.0, 1.0], [ -1.0, 1.0])
    known  = np.array([[0.6, 0.5], [0.6, 0.9], [0.7,0.5]])
    truth_offset = np.array([-0.4, -0.2])
    target3 = known + truth_offset
    plot_cost(known, target3)
    DE_result = differential_evolution(fn, bounds, args=(known, target3))
    print(DE_result)

    target = np.array([[ 8.147236863931789e-01, 1.576130816775483e-01],
                       [ 9.057919370756192e-01, 9.705927817606157e-01],
                       [ 2.0e-01              , 3.0e-01              ],
                       [ 9.133758561390194e-01, 4.853756487228412e-01],
                       [ 6.323592462254095e-01, 8.002804688888001e-01],
                       [ 2.0e-01              , 7.0e-01              ],
                       [ 2.784982188670484e-01, 4.217612826262750e-01],
                       [ 5.468815192049838e-01, 9.157355251890671e-01],
                       [ 9.575068354342976e-01, 7.922073295595544e-01],
                       [ 3.0e-01              , 3.0e-01              ]])

    plot_cost(known, target)

    DE_result = differential_evolution(fn, bounds, args=(known, target))
    print(DE_result)

if __name__ == "__main__": main()
